@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -25,12 +25,10 @@
 
 #include "SDL_stdinc.h"
 
-#if defined(SDL_LOADSO_DISABLED)
-#undef SDL_VIDEO_VULKAN
-#define SDL_VIDEO_VULKAN 0
-#endif
-
 #if SDL_VIDEO_VULKAN
+#if SDL_LOADSO_DISABLED || SDL_LOADSO_DUMMY
+#error You should not be here.
+#endif
 
 #if SDL_VIDEO_DRIVER_ANDROID
 #define VK_USE_PLATFORM_ANDROID_KHR
