@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -24,32 +24,34 @@
 
 #include "SDL_waylanddatamanager.h"
 #include "SDL_waylandevents_c.h"
+#include "SDL_waylandclipboard.h"
 
 int
 Wayland_SetClipboardText(_THIS, const char *text)
 {
     SDL_VideoData *video_data = NULL;
     SDL_WaylandDataDevice *data_device = NULL;
-    
+
     int status = 0;
- 
+
     if (_this == NULL || _this->driverdata == NULL) {
         status = SDL_SetError("Video driver uninitialized");
     } else {
         video_data = _this->driverdata;
-        /* TODO: Support more than one seat */ 
-        data_device = Wayland_get_data_device(video_data->input);
-        if (text[0] != '\0') {
-            SDL_WaylandDataSource* source = Wayland_data_source_create(_this);
-            Wayland_data_source_add_data(source, TEXT_MIME, text,
-                                         strlen(text) + 1); 
+        if (video_data->input != NULL && video_data->input->data_device != NULL) {
+            data_device = video_data->input->data_device;
+            if (text[0] != '\0') {
+                SDL_WaylandDataSource* source = Wayland_data_source_create(_this);
+                Wayland_data_source_add_data(source, TEXT_MIME, text,
+                                             SDL_strlen(text));
 
-            status = Wayland_data_device_set_selection(data_device, source);
-            if (status != 0) {
-                Wayland_data_source_destroy(source);
+                status = Wayland_data_device_set_selection(data_device, source);
+                if (status != 0) {
+                    Wayland_data_source_destroy(source);
+                }
+            } else {
+                status = Wayland_data_device_clear_selection(data_device);
             }
-        } else {
-            status = Wayland_data_device_clear_selection(data_device);
         }
     }
 
@@ -66,25 +68,27 @@ Wayland_GetClipboardText(_THIS)
 
     void *buffer = NULL;
     size_t length = 0;
- 
+
     if (_this == NULL || _this->driverdata == NULL) {
         SDL_SetError("Video driver uninitialized");
     } else {
         video_data = _this->driverdata;
-        /* TODO: Support more than one seat */ 
-        data_device = Wayland_get_data_device(video_data->input);
-        if (data_device->selection_offer != NULL) {
-            buffer = Wayland_data_offer_receive(data_device->selection_offer,
-                                                &length, TEXT_MIME, SDL_TRUE);
-            if (length > 0) {
-                text = (char*) buffer;
-            } 
-        } else if (data_device->selection_source != NULL) {
-            buffer = Wayland_data_source_get_data(data_device->selection_source,
-                                                  &length, TEXT_MIME, SDL_TRUE);
-            if (length > 0) {
-                text = (char*) buffer;
-            } 
+        if (video_data->input != NULL && video_data->input->data_device != NULL) {
+            data_device = video_data->input->data_device;
+            if (data_device->selection_offer != NULL) {
+                buffer = Wayland_data_offer_receive(data_device->selection_offer,
+                                                    &length, TEXT_MIME, SDL_TRUE);
+                if (length > 0) {
+                    text = (char*) buffer;
+                }
+            }
+            if (length == 0 && data_device->selection_source != NULL) {
+                buffer = Wayland_data_source_get_data(data_device->selection_source,
+                                                      &length, TEXT_MIME, SDL_TRUE);
+                if (length > 0) {
+                    text = (char*) buffer;
+                }
+            }
         }
     }
 
@@ -101,18 +105,20 @@ Wayland_HasClipboardText(_THIS)
     SDL_VideoData *video_data = NULL;
     SDL_WaylandDataDevice *data_device = NULL;
 
-    SDL_bool result = SDL_FALSE;    
+    SDL_bool result = SDL_FALSE;
     if (_this == NULL || _this->driverdata == NULL) {
         SDL_SetError("Video driver uninitialized");
     } else {
         video_data = _this->driverdata;
-        data_device = Wayland_get_data_device(video_data->input);
-        if (data_device != NULL && Wayland_data_offer_has_mime(
-                data_device->selection_offer, TEXT_MIME)) {
-            result = SDL_TRUE;
-        } else if(data_device != NULL && Wayland_data_source_has_mime(
-                data_device->selection_source, TEXT_MIME)) {
-            result = SDL_TRUE;
+        if (video_data->input != NULL && video_data->input->data_device != NULL) {
+            data_device = video_data->input->data_device;
+            if (Wayland_data_offer_has_mime(
+                    data_device->selection_offer, TEXT_MIME)) {
+                result = SDL_TRUE;
+            } else if (Wayland_data_source_has_mime(
+                    data_device->selection_source, TEXT_MIME)) {
+                result = SDL_TRUE;
+            }
         }
     }
     return result;
